@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>.
- * Copyright (C) 2008 Nokia Corporation, all rights reserved.
+ * Copyright (C) 2008 Nokia Corporation.
  *
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *                               <zeeshan.ali@nokia.com>
@@ -36,6 +36,8 @@
 
 typedef struct _RygelTestContentDir RygelTestContentDir;
 typedef struct _RygelTestContentDirClass RygelTestContentDirClass;
+#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
+#define _rygel_resource_info_unref0(var) ((var == NULL) ? NULL : (var = (rygel_resource_info_unref (var), NULL)))
 
 
 
@@ -48,13 +50,12 @@ void module_init (RygelPluginLoader* loader) {
 	RygelPlugin* plugin;
 	RygelResourceInfo* resource_info;
 	g_return_if_fail (loader != NULL);
-	plugin = rygel_plugin_new ("Test", "Test Streams");
-	/* We only implement a ContentDirectory service*/
+	plugin = rygel_plugin_new_MediaServer ("Test", "Test Streams");
 	resource_info = rygel_resource_info_new (RYGEL_CONTENT_DIRECTORY_UPNP_ID, RYGEL_CONTENT_DIRECTORY_UPNP_TYPE, RYGEL_CONTENT_DIRECTORY_DESCRIPTION_PATH, RYGEL_TYPE_TEST_CONTENT_DIR);
 	rygel_plugin_add_resource (plugin, resource_info);
 	rygel_plugin_loader_add_plugin (loader, plugin);
-	(plugin == NULL) ? NULL : (plugin = (g_object_unref (plugin), NULL));
-	(resource_info == NULL) ? NULL : (resource_info = (rygel_resource_info_unref (resource_info), NULL));
+	_g_object_unref0 (plugin);
+	_rygel_resource_info_unref0 (resource_info);
 }
 
 
