@@ -28,11 +28,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <gio/gio.h>
-#include <gee/arraylist.h>
-#include <gee/collection.h>
-#include <gee/iterable.h>
-#include <gee/iterator.h>
-#include <gee/list.h>
+#include <gee.h>
 
 
 #define RYGEL_TYPE_CONTENT_DIRECTORY (rygel_content_directory_get_type ())
@@ -95,6 +91,9 @@ typedef struct _RygelHTTPServerClass RygelHTTPServerClass;
 
 typedef struct _RygelBrowse RygelBrowse;
 typedef struct _RygelBrowseClass RygelBrowseClass;
+#define _g_free0(var) (var = (g_free (var), NULL))
+#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
+#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
 
 #define RYGEL_TYPE_STATE_MACHINE (rygel_state_machine_get_type ())
 #define RYGEL_STATE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RYGEL_TYPE_STATE_MACHINE, RygelStateMachine))
@@ -106,19 +105,11 @@ typedef struct _RygelStateMachineIface RygelStateMachineIface;
 typedef struct _RygelMediaObjectPrivate RygelMediaObjectPrivate;
 typedef struct _RygelMediaContainerPrivate RygelMediaContainerPrivate;
 
-/**
- * Errors used by ContentDirectory and deriving classes.
- */
 typedef enum  {
 	RYGEL_CONTENT_DIRECTORY_ERROR_NO_SUCH_OBJECT = 701,
 	RYGEL_CONTENT_DIRECTORY_ERROR_INVALID_ARGS = 402
 } RygelContentDirectoryError;
 #define RYGEL_CONTENT_DIRECTORY_ERROR rygel_content_directory_error_quark ()
-/**
- * Basic implementation of UPnP ContentDirectory service version 2. Most often
- * plugins will provide a child of this class. The inheriting classes should
- * override create_root_container method.
- */
 struct _RygelContentDirectory {
 	GUPnPService parent_instance;
 	RygelContentDirectoryPrivate * priv;
@@ -144,35 +135,28 @@ struct _RygelContentDirectoryPrivate {
 	GeeArrayList* browses;
 };
 
-/**
- * StateMachine interface.
- */
 struct _RygelStateMachineIface {
 	GTypeInterface parent_iface;
-	void (*run) (RygelStateMachine* self, GCancellable* cancellable);
+	void (*run) (RygelStateMachine* self);
+	GCancellable* (*get_cancellable) (RygelStateMachine* self);
+	void (*set_cancellable) (RygelStateMachine* self, GCancellable* value);
 };
 
-/**
- * Represents a media object (container and item).
- */
 struct _RygelMediaObject {
 	GObject parent_instance;
 	RygelMediaObjectPrivate * priv;
 	char* id;
 	char* title;
+	guint64 modified;
+	GeeArrayList* uris;
 	RygelMediaContainer* parent;
+	RygelMediaContainer* parent_ref;
 };
 
 struct _RygelMediaObjectClass {
 	GObjectClass parent_class;
 };
 
-/**
- * Represents a container (folder) for media items and containers. Provides
- * basic serialization (to DIDLLiteWriter) implementation. Deriving classes
- * are supposed to provide working implementations of get_children and
- * find_object.
- */
 struct _RygelMediaContainer {
 	RygelMediaObject parent_instance;
 	RygelMediaContainerPrivate * priv;
@@ -189,6 +173,7 @@ struct _RygelMediaContainerClass {
 };
 
 
+static gpointer rygel_content_directory_parent_class = NULL;
 
 GQuark rygel_content_directory_error_quark (void);
 GType rygel_content_directory_get_type (void);
@@ -206,8 +191,8 @@ enum  {
 #define RYGEL_CONTENT_DIRECTORY_DESCRIPTION_PATH "xml/ContentDirectory.xml"
 RygelMediaContainer* rygel_content_directory_create_root_container (RygelContentDirectory* self);
 static RygelMediaContainer* rygel_content_directory_real_create_root_container (RygelContentDirectory* self);
-RygelHTTPServer* rygel_http_server_new (RygelContentDirectory* content_dir, const char* name);
-RygelHTTPServer* rygel_http_server_construct (GType object_type, RygelContentDirectory* content_dir, const char* name);
+RygelHTTPServer* rygel_http_server_new (RygelContentDirectory* content_dir, const char* name, GError** error);
+RygelHTTPServer* rygel_http_server_construct (GType object_type, RygelContentDirectory* content_dir, const char* name, GError** error);
 static void rygel_content_directory_on_container_updated (RygelContentDirectory* self, RygelMediaContainer* root_container, RygelMediaContainer* updated_container);
 static void _rygel_content_directory_on_container_updated_rygel_media_container_container_updated (RygelMediaContainer* _sender, RygelMediaContainer* container, gpointer self);
 static void rygel_content_directory_browse_cb (RygelContentDirectory* self, RygelContentDirectory* content_dir, GUPnPServiceAction* action);
@@ -231,7 +216,7 @@ static void _rygel_content_directory_get_feature_list_cb_gupnp_service_action_in
 static void rygel_content_directory_query_feature_list (RygelContentDirectory* self, RygelContentDirectory* content_dir, const char* variable, GValue* value);
 static void _rygel_content_directory_query_feature_list_gupnp_service_query_variable (RygelContentDirectory* _sender, const char* variable, GValue* value, gpointer self);
 GType rygel_state_machine_get_type (void);
-void rygel_state_machine_run (RygelStateMachine* self, GCancellable* cancellable);
+void rygel_state_machine_run (RygelStateMachine* self);
 static void rygel_content_directory_real_constructed (GObject* base);
 RygelBrowse* rygel_browse_new (RygelContentDirectory* content_dir, GUPnPServiceAction* action);
 RygelBrowse* rygel_browse_construct (GType object_type, RygelContentDirectory* content_dir, GUPnPServiceAction* action);
@@ -243,8 +228,6 @@ static gboolean rygel_content_directory_update_notify (RygelContentDirectory* se
 static gboolean _rygel_content_directory_update_notify_gsource_func (gpointer self);
 RygelContentDirectory* rygel_content_directory_new (void);
 RygelContentDirectory* rygel_content_directory_construct (GType object_type);
-RygelContentDirectory* rygel_content_directory_new (void);
-static gpointer rygel_content_directory_parent_class = NULL;
 static void rygel_content_directory_finalize (GObject* obj);
 static int _vala_strcmp0 (const char * str1, const char * str2);
 
@@ -255,10 +238,11 @@ GQuark rygel_content_directory_error_quark (void) {
 }
 
 
-/* Public abstract methods derived classes need to implement*/
 static RygelMediaContainer* rygel_content_directory_real_create_root_container (RygelContentDirectory* self) {
+	RygelMediaContainer* result;
 	g_return_val_if_fail (self != NULL, NULL);
-	return NULL;
+	result = NULL;
+	return result;
 }
 
 
@@ -324,47 +308,63 @@ static void _rygel_content_directory_query_feature_list_gupnp_service_query_vari
 
 static void rygel_content_directory_real_constructed (GObject* base) {
 	RygelContentDirectory * self;
+	GError * _inner_error_;
 	GCancellable* _tmp0_;
 	RygelMediaContainer* _tmp1_;
-	RygelHTTPServer* _tmp2_;
-	GeeArrayList* _tmp3_;
 	GeeArrayList* _tmp4_;
-	char* _tmp5_;
+	GeeArrayList* _tmp5_;
 	char* _tmp6_;
 	char* _tmp7_;
+	char* _tmp8_;
 	self = (RygelContentDirectory*) base;
-	_tmp0_ = NULL;
-	self->cancellable = (_tmp0_ = g_cancellable_new (), (self->cancellable == NULL) ? NULL : (self->cancellable = (g_object_unref (self->cancellable), NULL)), _tmp0_);
-	_tmp1_ = NULL;
-	self->root_container = (_tmp1_ = rygel_content_directory_create_root_container (self), (self->root_container == NULL) ? NULL : (self->root_container = (g_object_unref (self->root_container), NULL)), _tmp1_);
-	_tmp2_ = NULL;
-	self->http_server = (_tmp2_ = rygel_http_server_new (self, g_type_name (G_TYPE_FROM_INSTANCE ((GObject*) self))), (self->http_server == NULL) ? NULL : (self->http_server = (g_object_unref (self->http_server), NULL)), _tmp2_);
-	_tmp3_ = NULL;
-	self->priv->browses = (_tmp3_ = gee_array_list_new (RYGEL_TYPE_BROWSE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), (self->priv->browses == NULL) ? NULL : (self->priv->browses = (g_object_unref (self->priv->browses), NULL)), _tmp3_);
-	_tmp4_ = NULL;
-	self->priv->updated_containers = (_tmp4_ = gee_array_list_new (RYGEL_TYPE_MEDIA_CONTAINER, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), (self->priv->updated_containers == NULL) ? NULL : (self->priv->updated_containers = (g_object_unref (self->priv->updated_containers), NULL)), _tmp4_);
+	_inner_error_ = NULL;
+	self->cancellable = (_tmp0_ = g_cancellable_new (), _g_object_unref0 (self->cancellable), _tmp0_);
+	self->root_container = (_tmp1_ = rygel_content_directory_create_root_container (self), _g_object_unref0 (self->root_container), _tmp1_);
+	{
+		RygelHTTPServer* _tmp2_;
+		RygelHTTPServer* _tmp3_;
+		_tmp2_ = rygel_http_server_new (self, g_type_name (G_TYPE_FROM_INSTANCE ((GObject*) self)), &_inner_error_);
+		if (_inner_error_ != NULL) {
+			goto __catch17_g_error;
+			goto __finally17;
+		}
+		self->http_server = (_tmp3_ = _tmp2_, _g_object_unref0 (self->http_server), _tmp3_);
+	}
+	goto __finally17;
+	__catch17_g_error:
+	{
+		GError * err;
+		err = _inner_error_;
+		_inner_error_ = NULL;
+		{
+			g_critical ("rygel-content-directory.vala:77: Failed to create HTTP server for %s: %s", g_type_name (G_TYPE_FROM_INSTANCE ((GObject*) self)), err->message);
+			_g_error_free0 (err);
+			return;
+		}
+	}
+	__finally17:
+	if (_inner_error_ != NULL) {
+		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
+		g_clear_error (&_inner_error_);
+		return;
+	}
+	self->priv->browses = (_tmp4_ = gee_array_list_new (RYGEL_TYPE_BROWSE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), _g_object_unref0 (self->priv->browses), _tmp4_);
+	self->priv->updated_containers = (_tmp5_ = gee_array_list_new (RYGEL_TYPE_MEDIA_CONTAINER, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), _g_object_unref0 (self->priv->updated_containers), _tmp5_);
 	g_signal_connect_object (self->root_container, "container-updated", (GCallback) _rygel_content_directory_on_container_updated_rygel_media_container_container_updated, self, 0);
-	_tmp5_ = NULL;
-	self->feature_list = (_tmp5_ = g_strdup ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" "<Features xmlns=\"urn:schemas-upnp-org:av:avs\" " "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " "xsi:schemaLocation=\"urn:schemas-upnp-org:av:avs" "http://www.upnp.org/schemas/av/avs-v1-20060531.xsd\">" "</Features>"), self->feature_list = (g_free (self->feature_list), NULL), _tmp5_);
-	_tmp6_ = NULL;
-	self->search_caps = (_tmp6_ = g_strdup (""), self->search_caps = (g_free (self->search_caps), NULL), _tmp6_);
-	_tmp7_ = NULL;
-	self->sort_caps = (_tmp7_ = g_strdup (""), self->sort_caps = (g_free (self->sort_caps), NULL), _tmp7_);
+	self->feature_list = (_tmp6_ = g_strdup ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" "<Features xmlns=\"urn:schemas-upnp-org:av:avs\" " "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " "xsi:schemaLocation=\"urn:schemas-upnp-org:av:avs" "http://www.upnp.org/schemas/av/avs-v1-20060531.xsd\">" "</Features>"), _g_free0 (self->feature_list), _tmp6_);
+	self->search_caps = (_tmp7_ = g_strdup (""), _g_free0 (self->search_caps), _tmp7_);
+	self->sort_caps = (_tmp8_ = g_strdup (""), _g_free0 (self->sort_caps), _tmp8_);
 	g_signal_connect_object ((GUPnPService*) self, "action-invoked::Browse", (GCallback) _rygel_content_directory_browse_cb_gupnp_service_action_invoked, self, 0);
-	/* Connect SystemUpdateID related signals */
 	g_signal_connect_object ((GUPnPService*) self, "action-invoked::GetSystemUpdateID", (GCallback) _rygel_content_directory_get_system_update_id_cb_gupnp_service_action_invoked, self, 0);
 	g_signal_connect_object ((GUPnPService*) self, "query-variable::SystemUpdateID", (GCallback) _rygel_content_directory_query_system_update_id_gupnp_service_query_variable, self, 0);
 	g_signal_connect_object ((GUPnPService*) self, "query-variable::ContainerUpdateIDs", (GCallback) _rygel_content_directory_query_container_update_ids_gupnp_service_query_variable, self, 0);
-	/* Connect SearchCapabilities related signals */
 	g_signal_connect_object ((GUPnPService*) self, "action-invoked::GetSearchCapabilities", (GCallback) _rygel_content_directory_get_search_capabilities_cb_gupnp_service_action_invoked, self, 0);
 	g_signal_connect_object ((GUPnPService*) self, "query-variable::SearchCapabilities", (GCallback) _rygel_content_directory_query_search_capabilities_gupnp_service_query_variable, self, 0);
-	/* Connect SortCapabilities related signals */
 	g_signal_connect_object ((GUPnPService*) self, "action-invoked::GetSortCapabilities", (GCallback) _rygel_content_directory_get_sort_capabilities_cb_gupnp_service_action_invoked, self, 0);
 	g_signal_connect_object ((GUPnPService*) self, "query-variable::SortCapabilities", (GCallback) _rygel_content_directory_query_sort_capabilities_gupnp_service_query_variable, self, 0);
-	/* Connect FeatureList related signals */
 	g_signal_connect_object ((GUPnPService*) self, "action-invoked::GetFeatureList", (GCallback) _rygel_content_directory_get_feature_list_cb_gupnp_service_action_invoked, self, 0);
 	g_signal_connect_object ((GUPnPService*) self, "query-variable::FeatureList", (GCallback) _rygel_content_directory_query_feature_list_gupnp_service_query_variable, self, 0);
-	rygel_state_machine_run ((RygelStateMachine*) self->http_server, self->cancellable);
+	rygel_state_machine_run ((RygelStateMachine*) self->http_server);
 }
 
 
@@ -373,17 +373,16 @@ static void _rygel_content_directory_on_browse_completed_rygel_state_machine_com
 }
 
 
-/* Browse action implementation */
 static void rygel_content_directory_real_browse_cb (RygelContentDirectory* self, RygelContentDirectory* content_dir, GUPnPServiceAction* action) {
 	RygelBrowse* browse;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (action != NULL);
 	browse = rygel_browse_new (self, action);
-	gee_collection_add ((GeeCollection*) self->priv->browses, browse);
+	gee_abstract_collection_add ((GeeAbstractCollection*) self->priv->browses, browse);
 	g_signal_connect_object ((RygelStateMachine*) browse, "completed", (GCallback) _rygel_content_directory_on_browse_completed_rygel_state_machine_completed, self, 0);
-	rygel_state_machine_run ((RygelStateMachine*) browse, self->cancellable);
-	(browse == NULL) ? NULL : (browse = (g_object_unref (browse), NULL));
+	rygel_state_machine_run ((RygelStateMachine*) browse);
+	_g_object_unref0 (browse);
 }
 
 
@@ -392,103 +391,85 @@ void rygel_content_directory_browse_cb (RygelContentDirectory* self, RygelConten
 }
 
 
-/* GetSystemUpdateID action implementation */
 static void rygel_content_directory_get_system_update_id_cb (RygelContentDirectory* self, RygelContentDirectory* content_dir, GUPnPServiceAction* action) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (action != NULL);
-	/* Set action return arguments */
 	gupnp_service_action_set (action, "Id", G_TYPE_UINT, self->system_update_id, NULL);
 	gupnp_service_action_return (action);
 }
 
 
-/* Query GetSystemUpdateID */
 static void rygel_content_directory_query_system_update_id (RygelContentDirectory* self, RygelContentDirectory* content_dir, const char* variable, GValue* value) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (variable != NULL);
-	/* Set action return arguments */
 	g_value_init (&(*value), G_TYPE_UINT);
 	g_value_set_uint (&(*value), (guint) self->system_update_id);
 }
 
 
-/* Query ContainerUpdateIDs */
 static void rygel_content_directory_query_container_update_ids (RygelContentDirectory* self, RygelContentDirectory* content_dir, const char* variable, GValue* value) {
 	char* update_ids;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (variable != NULL);
 	update_ids = rygel_content_directory_create_container_update_ids (self);
-	/* Set action return arguments */
 	g_value_init (&(*value), G_TYPE_STRING);
 	g_value_set_string (&(*value), update_ids);
-	update_ids = (g_free (update_ids), NULL);
+	_g_free0 (update_ids);
 }
 
 
-/* action GetSearchCapabilities implementation */
 static void rygel_content_directory_get_search_capabilities_cb (RygelContentDirectory* self, RygelContentDirectory* content_dir, GUPnPServiceAction* action) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (action != NULL);
-	/* Set action return arguments */
 	gupnp_service_action_set (action, "SearchCaps", G_TYPE_STRING, self->search_caps, NULL);
 	gupnp_service_action_return (action);
 }
 
 
-/* Query SearchCapabilities */
 static void rygel_content_directory_query_search_capabilities (RygelContentDirectory* self, RygelContentDirectory* content_dir, const char* variable, GValue* value) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (variable != NULL);
-	/* Set action return arguments */
 	g_value_init (&(*value), G_TYPE_STRING);
 	g_value_set_string (&(*value), self->search_caps);
 }
 
 
-/* action GetSortCapabilities implementation */
 static void rygel_content_directory_get_sort_capabilities_cb (RygelContentDirectory* self, RygelContentDirectory* content_dir, GUPnPServiceAction* action) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (action != NULL);
-	/* Set action return arguments */
 	gupnp_service_action_set (action, "SortCaps", G_TYPE_STRING, self->sort_caps, NULL);
 	gupnp_service_action_return (action);
 }
 
 
-/* Query SortCapabilities */
 static void rygel_content_directory_query_sort_capabilities (RygelContentDirectory* self, RygelContentDirectory* content_dir, const char* variable, GValue* value) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (variable != NULL);
-	/* Set action return arguments */
 	g_value_init (&(*value), G_TYPE_STRING);
 	g_value_set_string (&(*value), self->sort_caps);
 }
 
 
-/* action GetFeatureList implementation */
 static void rygel_content_directory_get_feature_list_cb (RygelContentDirectory* self, RygelContentDirectory* content_dir, GUPnPServiceAction* action) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (action != NULL);
-	/* Set action return arguments */
 	gupnp_service_action_set (action, "FeatureList", G_TYPE_STRING, self->feature_list, NULL);
 	gupnp_service_action_return (action);
 }
 
 
-/* Query FeatureList */
 static void rygel_content_directory_query_feature_list (RygelContentDirectory* self, RygelContentDirectory* content_dir, const char* variable, GValue* value) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (content_dir != NULL);
 	g_return_if_fail (variable != NULL);
-	/* Set action return arguments */
 	g_value_init (&(*value), G_TYPE_STRING);
 	g_value_set_string (&(*value), self->feature_list);
 }
@@ -497,42 +478,42 @@ static void rygel_content_directory_query_feature_list (RygelContentDirectory* s
 static void rygel_content_directory_on_browse_completed (RygelContentDirectory* self, RygelBrowse* browse) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (browse != NULL);
-	gee_collection_remove ((GeeCollection*) self->priv->browses, browse);
+	gee_abstract_collection_remove ((GeeAbstractCollection*) self->priv->browses, browse);
 }
 
 
 static char* rygel_content_directory_create_container_update_ids (RygelContentDirectory* self) {
+	char* result;
 	char* update_ids;
 	g_return_val_if_fail (self != NULL, NULL);
 	update_ids = g_strdup ("");
 	{
 		GeeIterator* _container_it;
-		_container_it = gee_iterable_iterator ((GeeIterable*) self->priv->updated_containers);
-		while (gee_iterator_next (_container_it)) {
+		_container_it = gee_abstract_collection_iterator ((GeeAbstractCollection*) self->priv->updated_containers);
+		while (TRUE) {
 			RygelMediaContainer* container;
 			char* _tmp4_;
 			char* _tmp3_;
 			char* _tmp2_;
 			char* _tmp1_;
+			if (!gee_iterator_next (_container_it)) {
+				break;
+			}
 			container = (RygelMediaContainer*) gee_iterator_get (_container_it);
 			if (_vala_strcmp0 (update_ids, "") != 0) {
 				char* _tmp0_;
-				_tmp0_ = NULL;
-				update_ids = (_tmp0_ = g_strconcat (update_ids, ",", NULL), update_ids = (g_free (update_ids), NULL), _tmp0_);
+				update_ids = (_tmp0_ = g_strconcat (update_ids, ",", NULL), _g_free0 (update_ids), _tmp0_);
 			}
-			_tmp4_ = NULL;
-			_tmp3_ = NULL;
-			_tmp2_ = NULL;
-			_tmp1_ = NULL;
-			update_ids = (_tmp4_ = g_strconcat (update_ids, _tmp3_ = g_strconcat (_tmp1_ = g_strconcat (((RygelMediaObject*) container)->id, ",", NULL), _tmp2_ = g_strdup_printf ("%u", container->update_id), NULL), NULL), update_ids = (g_free (update_ids), NULL), _tmp4_);
-			_tmp3_ = (g_free (_tmp3_), NULL);
-			_tmp2_ = (g_free (_tmp2_), NULL);
-			_tmp1_ = (g_free (_tmp1_), NULL);
-			(container == NULL) ? NULL : (container = (g_object_unref (container), NULL));
+			update_ids = (_tmp4_ = g_strconcat (update_ids, _tmp3_ = g_strconcat (_tmp1_ = g_strconcat (((RygelMediaObject*) container)->id, ",", NULL), _tmp2_ = g_strdup_printf ("%u", container->update_id), NULL), NULL), _g_free0 (update_ids), _tmp4_);
+			_g_free0 (_tmp3_);
+			_g_free0 (_tmp2_);
+			_g_free0 (_tmp1_);
+			_g_object_unref0 (container);
 		}
-		(_container_it == NULL) ? NULL : (_container_it = (g_object_unref (_container_it), NULL));
+		_g_object_unref0 (_container_it);
 	}
-	return update_ids;
+	result = update_ids;
+	return result;
 }
 
 
@@ -541,53 +522,38 @@ static gboolean _rygel_content_directory_update_notify_gsource_func (gpointer se
 }
 
 
-/**
-     * handler for container_updated signal on root_container. We don't
-     * immediately send the notification for changes but schedule the
-     * notification after 0.2 seconds. Also we don't clear the updated
-     * container list immediately after notification but rather in this
-     * function. Please refer to ContentDirectory version 2 specs for details
-     * on why we do all this the way we do.
-     *
-     * @param root_container the root_container
-     * @param updated_container the container that just got updated
-     */
 static void rygel_content_directory_on_container_updated (RygelContentDirectory* self, RygelMediaContainer* root_container, RygelMediaContainer* updated_container) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (root_container != NULL);
 	g_return_if_fail (updated_container != NULL);
 	self->system_update_id++;
 	if (self->priv->clear_updated_containers) {
-		gee_collection_clear ((GeeCollection*) self->priv->updated_containers);
+		gee_abstract_collection_clear ((GeeAbstractCollection*) self->priv->updated_containers);
 		self->priv->clear_updated_containers = FALSE;
 	}
-	/* UPnP specs dicate we make sure only last update be evented*/
-	gee_collection_remove ((GeeCollection*) self->priv->updated_containers, updated_container);
-	gee_collection_add ((GeeCollection*) self->priv->updated_containers, updated_container);
+	gee_abstract_collection_remove ((GeeAbstractCollection*) self->priv->updated_containers, updated_container);
+	gee_abstract_collection_add ((GeeAbstractCollection*) self->priv->updated_containers, updated_container);
 	if (self->priv->update_notify_id == 0) {
-		self->priv->update_notify_id = g_timeout_add ((guint) 200, _rygel_content_directory_update_notify_gsource_func, self);
+		self->priv->update_notify_id = g_timeout_add_full (G_PRIORITY_DEFAULT, (guint) 200, _rygel_content_directory_update_notify_gsource_func, g_object_ref (self), g_object_unref);
 	}
 }
 
 
 static gboolean rygel_content_directory_update_notify (RygelContentDirectory* self) {
+	gboolean result;
 	char* update_ids;
-	gboolean _tmp0_;
 	g_return_val_if_fail (self != NULL, FALSE);
 	update_ids = rygel_content_directory_create_container_update_ids (self);
 	gupnp_service_notify ((GUPnPService*) self, "ContainerUpdateIDs", G_TYPE_STRING, update_ids, NULL);
 	gupnp_service_notify ((GUPnPService*) self, "SystemUpdateID", G_TYPE_UINT, self->system_update_id, NULL);
 	self->priv->clear_updated_containers = TRUE;
 	self->priv->update_notify_id = (guint) 0;
-	return (_tmp0_ = FALSE, update_ids = (g_free (update_ids), NULL), _tmp0_);
+	result = FALSE;
+	_g_free0 (update_ids);
+	return result;
 }
 
 
-/**
- * Basic implementation of UPnP ContentDirectory service version 2. Most often
- * plugins will provide a child of this class. The inheriting classes should
- * override create_root_container method.
- */
 RygelContentDirectory* rygel_content_directory_construct (GType object_type) {
 	RygelContentDirectory * self;
 	self = g_object_newv (object_type, 0, NULL);
@@ -619,17 +585,16 @@ static void rygel_content_directory_finalize (GObject* obj) {
 	RygelContentDirectory * self;
 	self = RYGEL_CONTENT_DIRECTORY (obj);
 	{
-		/* Cancel all state machines*/
 		g_cancellable_cancel (self->cancellable);
 	}
-	self->feature_list = (g_free (self->feature_list), NULL);
-	self->search_caps = (g_free (self->search_caps), NULL);
-	self->sort_caps = (g_free (self->sort_caps), NULL);
-	(self->http_server == NULL) ? NULL : (self->http_server = (g_object_unref (self->http_server), NULL));
-	(self->root_container == NULL) ? NULL : (self->root_container = (g_object_unref (self->root_container), NULL));
-	(self->priv->updated_containers == NULL) ? NULL : (self->priv->updated_containers = (g_object_unref (self->priv->updated_containers), NULL));
-	(self->priv->browses == NULL) ? NULL : (self->priv->browses = (g_object_unref (self->priv->browses), NULL));
-	(self->cancellable == NULL) ? NULL : (self->cancellable = (g_object_unref (self->cancellable), NULL));
+	_g_free0 (self->feature_list);
+	_g_free0 (self->search_caps);
+	_g_free0 (self->sort_caps);
+	_g_object_unref0 (self->http_server);
+	_g_object_unref0 (self->root_container);
+	_g_object_unref0 (self->priv->updated_containers);
+	_g_object_unref0 (self->priv->browses);
+	_g_object_unref0 (self->cancellable);
 	G_OBJECT_CLASS (rygel_content_directory_parent_class)->finalize (obj);
 }
 
